/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.editor.view.spi;

import javax.swing.text.Element;
import javax.swing.text.View;
import org.netbeans.lib.editor.view.ViewUtilitiesImpl;

/**
 * Various utility methods related to views.
 *
 * @author Miloslav Metelka
 * @version 1.00
 */

public class ViewUtilities {
    
    private ViewUtilities() {
    }
    

    /**
     * Create view that will cover the whole document.
     */
    public static View createDocumentView(Element elem) {
        return new org.netbeans.lib.editor.view.GapDocumentView(elem);
    }

    /**
     * Check correctness of the hierarchy under the given view.
     * <br>
     * Current checks:
     * <ul>
     *     <li>
     *         Children are Parents are checked to have correct parent info.
     * </ul>
     */
    public static void checkViewHierarchy(View v) {
        ViewUtilitiesImpl.checkViewHierarchy(v);
    }

    /**
     * Test whether the axis is valid.
     *
     * @param axis integer axis
     * @return true if the axis is either <code>View.X_AXIS</code>
     *  or <code>View.Y_AXIS</code>. False is returned otherwise.
     */
    public static boolean isAxisValid(int axis) {
        switch (axis) {
            case View.X_AXIS:
            case View.Y_AXIS:
                return true;
        }
        
        return false;
    }

}
